import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;

import javax.imageio.ImageIO;
import javax.swing.JPanel;

public class Grid extends JPanel implements MouseListener {
	private static final long serialVersionUID = 1L;
	private BufferedImage gridImage;
	private Object[][] array;
	public static final int X_ORIGIN = 54; // coordenada X arriba la izquierda
	public static final int Y_ORIGIN = 56; // coordenada Y arriba la izquierda
	public static final int TILE_SIZE = 47; // tamao del titulo 
	public static final int BORDER_SIZE = 5; // tamao del borde entre espacios
	private volatile boolean isTurn;
	private boolean state;

	//Constructor default
	public Grid() {
		this(new Object[10][10], "gridLabels.png");
	}

	//Constructor que toma un aray
	public Grid(Object[][] arr) {
		this(arr, "gridLabels.png");
	}

	//Constructor con array y archivo
	public Grid(Object[][] arr, String path) {
		array = arr;
		isTurn = true;
		state = false;
		// pone el fondo blanco y asigna tamaos
		setBackground(Color.white);
		setPreferredSize(new Dimension((X_ORIGIN+ arr.length + 1 + ((TILE_SIZE+BORDER_SIZE)*array.length)), 
				Y_ORIGIN+ arr.length + 1 + ((TILE_SIZE+BORDER_SIZE)*array.length)));
		setSize(getPreferredSize());
		setLocation(0,0);

		try {
			gridImage = ImageIO.read(new File(path));
		} catch (IOException e) {
			System.out.println("Fallo cargar la imagen");
		}
	}

	@Override
	public void paintComponent(Graphics g) {
		super.paintComponent(g);
		Graphics2D g2 = (Graphics2D) g;
		// dibuja las celdas
		g2.drawImage(gridImage, 0, 0, this);

		// administra lo que sucede dentro de una celda
		for (int i = 0; i < array.length; i++) {
			for (int j = 0; j < array[i].length; j++) {
				// checkea si una pieza de barco no ha sido destruido 
				// en caso que la celda no este vacia
				if (array[i][j].equals((Object) 1) || ((array[i][j]).getClass().getName().equals("ShipPiece")
						&& !((ShipPiece) array[i][j]).isDestroy())) {
					// cubre la celda de un fondo gris
					g2.setColor(Color.gray);
					g2.fillRect(X_ORIGIN + i + 1 + ((TILE_SIZE + BORDER_SIZE) * i), Y_ORIGIN + j + 1 + ((TILE_SIZE + BORDER_SIZE) * j),
							TILE_SIZE+(BORDER_SIZE/2)-1, TILE_SIZE+(BORDER_SIZE/2)-1);
					// si hay una pieza de barco en la posicion, esta es destruida
				} else if ((array[i][j]).getClass().getName().equals("ShipPiece")) {
					// pone la imagen correspondiente a la pieza de barco
					g2.drawImage(((ShipPiece) array[i][j]).getShipImage(),
							X_ORIGIN + i + ((TILE_SIZE + BORDER_SIZE) * i) + BORDER_SIZE/2,
							Y_ORIGIN + j + ((TILE_SIZE + BORDER_SIZE) * j) + BORDER_SIZE/2, this);
				}
			}
		}

	}

	@Override
	public void mouseReleased(MouseEvent e) {
		// click izquierdo
		if (isTurn && e.getButton() == MouseEvent.BUTTON1) {

			// pone la cordenada X marcada por el mouse en el arreglo de grids (celdas)
			int value = e.getX();
			int counter1 = 0;
			while (X_ORIGIN + ((TILE_SIZE + BORDER_SIZE) * counter1) + BORDER_SIZE < value) {
				counter1++;
			}
			counter1--;

			// pone la cordenada Y marcada por el mouse en el arreglo de grids (celdas)
			int value2 = e.getY() - (TILE_SIZE / 2);
			int counter2 = 0;
			while (Y_ORIGIN + ((TILE_SIZE + BORDER_SIZE) * counter2) + BORDER_SIZE < value2) {
				counter2++;
			}
			counter2--;

			// si es una posicion valida dentro del array
			if (counter1 < array.length && counter1 >= 0) {
				if (counter2 < array[0].length && counter2 >= 0) {
					// si el objeto de la coordenada es 1
					if (array[counter1][counter2].equals((Object) 1)) {
						// pone el objeto en la coordenada como 0 (coordenada vacia)
						array[counter1][counter2] = 0;
						repaint();
						// termina el turno
						isTurn = false;
						// si en la coordenada hay una pieza de barco no destruida
					} else if ((array[counter1][counter2]).getClass().getName().equals("ShipPiece")
							&& !((ShipPiece) array[counter1][counter2]).isDestroy()) {
						// destruye la pieza del barco
						((ShipPiece) array[counter1][counter2]).destroy();
						repaint();
						// termina el turno
						isTurn = false;
					}
					state = false;
				}
			}
		}else if(!isTurn && e.getButton() == MouseEvent.BUTTON1){
			state = true;
		}
	}

	@Override
	public void mouseClicked(MouseEvent e) {
	}

	@Override
	public void mouseEntered(MouseEvent e) {
	}

	@Override
	public void mouseExited(MouseEvent e) {
	}

	@Override
	public void mousePressed(MouseEvent e) {
	}

	/*
	 * retorna isTurn
	 */
	public boolean isTurn() {
		return isTurn;
	}

	/*
	 * le pone el parametro del turno
	 */
	public void setTurn(boolean t) { //true, para jugador 1
		isTurn = t;
	}

	/*
	 * devuelve el arreglo de grids
	 */
	public Object[][] getArray() {
		return array;
	}

	/*
	 * leda un parametro al arreglo de grids
	 */
	public void setArray(Object[][] arr) {
		array = arr;
	}
	
	public boolean getState(){
		return state;
	}
	
	public void setState(boolean s){
		state = s;
	}

}
